; Arabic locale

00:00=رمانة‫
00:01=عنقودية‫
00:02=صاروخ‫
00:03=صاروخ موجه‫
00:04=بندقية
00:05=مطرقة
00:06=ترك
00:07=حبل
00:08=لغم
00:09=مسدس حربي
00:10=ديناميت
00:11=مضرب بيسبول
00:12=Shoryuken
00:13=ثانية
00:14=برشوت
00:15=هجوم جوي
00:16=هجوم الغام
00:17=شعلة
00:18=بناء
00:19=نقل
00:20=تبديل
00:21=هاون
00:22=سوط
00:23=انتحار
00:24=كيك
00:25=اغواء
00:26=ركية
00:27=قنبلة جحيم
00:28=صاروخ حفر
00:29=مسدس كرات
00:30=حارق
00:31=طيارة مسيطرة عن بعد
00:32=جاذبية قليلة
00:33=تاثير مضاعف
00:34=غير قابل للاضرار
00:35=وقت اضافي
00:36=منظار ليزر
00:37=مصاص دماء
00:38=قناص
00:39=صحن طائر
00:40=زجاجة حارقة
00:41=طير

; 01:00=Loading …
01:01=تعادل
01:02=%1 يفوز
01:03=جزء %1%
01:04=توقف
01:05=خروج (%1/%2)?
01:06=موت مفاجئ
01:07=%1 متبقي
01:08=طاقة: %1%
01:09=تحديث...
01:10=استعمال هذه لن ينهي دورك
01:11=This weapon or utility is not yet available!

; Event messages
; Hog (%1) died
02:00=%1 has kicked the bucket!
02:00=%1 has seen the light!
02:00=%1 never saw that coming!
02:00=%1 waves goodbye!
02:00=%1 has gone to a better place!
02:00=%1 meets his maker!
02:00=%1 can hang on no longer!
02:00=%1 has done his duty!
02:00=%1 makes the ultimate sacrifice!
02:00=%1 departs this mortal coil!
02:00=%1 makes like a tree and leaves!
02:00=%1 has timed out!
02:00=%1 says peace out!
02:00=%1 will be fondly remembered!
02:00=%1 has an aneurysm!
02:00=%1 leaves behind a wife and child
02:00=%1 has launched his last bazooka
02:00=%1 has tossed his last grenade
02:00=%1 has baked his last cake
02:00=%1 has swung on his last rope
02:00=%1 has called his last airstrike
02:00=%1 has pumped his last shotgun
02:00=%1 has thrown his last melon
02:00=%1 has drawn his last deagle
02:00=%1 took one shot too many
02:00=%1 could really have used a health crate
02:00=%1 has gone to play a better game
02:00=%1 has ragequit life
02:00=%1 fails
02:00=Poor poor %1...
02:00=%1 prefers wormux
02:00=%1 has been blocking shots with his face
02:00=%1 is a hero amongst me...err..hogs
02:00=%1 finds his place in Valhalla
02:00=%1 has left the building
02:00=%1 goes the way of the dinosaurs
02:00=%1 brings hedgehogs one step closer to extinction
02:00=%1 brings a tear to my eye
02:00=%1 is an ex-hog
02:00=%1 is pushing up the daisies
02:00=%1 has ceased to be
02:00=Say goodbye to %1
02:00=No hope left for %1
02:00=%1 faces the final curtain
02:00=Smoke 'em if you got 'em, %1
02:00=%1 suffers a Spontaneous Massive Existence Failure
02:00=%1 has passed on
02:00=%1 is stone dead
02:00=%1 is no more
02:00=%1 has expired
02:00=Bereft of life, %1 rests in peace
02:00=%1 joins the choir invisible
02:00=Farewell %1, we hardly knew ye!
02:00=%1 had a low tolerance for being shot
02:00=%1 could have used an extra life
02:00=Is there a doctor in the house?

; Hog (%1) drowned
02:01=%1 plays submarine!
02:01=%1 mimics the Titanic!
02:01=%1 swims like a stone!
02:01=%1 floats like a brick!
02:01=%1 checks out the deep end
02:01=%1 goes glug glug glug
02:01=%1 goes splash
02:01=%1 forgot his armbands
02:01=%1 really should have taken swimming lessons
02:01=%1 left his surfboard at home
02:01=%1 is washed up
02:01=%1 is one soggy hog
02:01=%1 forgot to bring his life jacket
02:01=%1 goes splish splash splish
02:01=%1 is sleeping with the fishes
02:01=%1 thinks the water physics suck in this game
02:01=%1 looks thirsty
02:01=the sea claims %1
02:01=%1 is lost at sea
02:01=%1 should have brought his scuba gear
02:01=%1 gets a burial at sea
02:01=%1 has that sinking feeling
02:01=%1 is practicing his backstroke
02:01=%1 goes in search of the Titanic
02:01=%1 is not Jesus
02:01=%1 is finding Nemo
02:01=%1 springs a leak
02:01=You've gotta wonder how many hogs are down there
02:01=%1 makes the ocean slightly higher
02:01=%1 didn't enlist in the Navy
02:01=%1 is doing his impersonation of a dead fish
02:01=At least you didn't go down the toilet, %1
02:01=Sonic couldn't swim and neither can %1
02:01=%1 wants to play Ecco the dolphin
02:01=%1 has gone to visit Aquaria
02:01=%1 has found the lost city of Atlantis
02:01=Your doggy paddle could use a little work, %1
02:01=%1 should have brought a jet ski
02:01=%1 doesn't like watersports
02:01=%1 is forever blowing bubbles
02:01=%1 is short of a raft
02:01=%1 thinks salt water is good for the skin
02:01=%1 gets salt water in his wounds
02:01=%1 has walked the plank
02:01=%1 has a bath
02:01=%1 is wet wet wet
02:01=%1 gets his quills wet
02:01=It's Davy Jones' locker for %1

; Round starts
02:02=Let's fight!
02:02=Armed and ready!
02:02=Let's get ready to rumble!
02:02=Let's get it on!
02:02=Let's get this party started
02:02=Last hog standing wins
02:02=Let's go!
02:02=Let's rock!
02:02=Let's jam!
02:02=It's beginning...
02:02=This is the start of something big
02:02=Welcome to Hedgewars
02:02=Welcome to the front lines
02:02=Crush your enemies!
02:02=May the best hog win
02:02=Victory or death
02:02=To the victor goes the spoils
02:02=Losing is not an option
02:02=Cry havoc! Let loose the hogs of war!
02:02=Hedgewars, brought to you by Hedgewars.org
02:02=GL HF
02:02=Just count yourself lucky you're not up against Tiyuri
02:02=Just count yourself lucky you're not up against unC0Rr
02:02=Just count yourself lucky you're not up against Nemo
02:02=Just count yourself lucky you're not up against Smaxx
02:02=Just count yourself lucky you're not up against Jessor
02:02=Give it your all!
02:02=The losers do the cleaning up!
02:02=Let the fight of the millenium begin
02:02=Let the fight of the century begin
02:02=Let the fight of the decade begin
02:02=Let the fight of the year begin
02:02=Let the fight of the month begin
02:02=Let the fight of the week begin
02:02=Let the fight of the day begin
02:02=Let the fight of the hour begin
02:02=Do your best!
02:02=Destroy the enemy!
02:02=Good luck
02:02=Have fun
02:02=Fight the good fight
02:02=Fight dirty
02:02=Fight with honour
02:02=Don't give up
02:02=Never surrender
02:02=Rock 'em and sock 'em!
02:02=Let the fragfest begin!
02:02=I hope you're ready for a tussle!
02:02=Go Go Go!
02:02=Hedgehogs advance!
02:02=Bring it to them!
02:02=Have no fear!
02:02=Be brave and conquer

; Round ends and team/clan (%1) wins
02:03=%1 يفوز

; Round ends in a draw
02:04=تعادل

; New health crate
02:05=Incoming aid!
02:05=Medic!
02:05=First aid from the skies!
02:05=A health pack for you
02:05=Good health.. in box form!
02:05=The doctor calls
02:05=Fresh band-aids!
02:05=This will make you feel better
02:05=A Hi-Potion! Whoops wrong game
02:05=A pick-me-up!
02:05=Grab it
02:05=A healthy snack
02:05=A remedy to pain
02:05=Correct Dosage: as many as you can find!
02:05=Urgent delivery
02:05=Supplies!

; New ammo crate
02:06=More weapons!
02:06=Reinforcements!
02:06=Lock and load!
02:06=I wonder what weapon is in there?
02:06=Supplies!
02:06=What could be inside?
02:06=Christmas comes early in Hedgewars
02:06=A present!
02:06=Special delivery!
02:06=It was a nightmare getting this through customs
02:06=Destructive toys from the heavens
02:06=Warning! Contents Volatile
02:06=Pick it up or blow it up, choice is yours
02:06=Goodies!
02:06=Mmmmm Ammo
02:06=A box of destructive power
02:06=Airmail!
02:06=Whatever's in that box, it ain't pizza
02:06=Get it!
02:06=Weapon drop incoming
02:06=Don't let the enemy grab that!
02:06=Shiny new toys!
02:06=A mysterious box!

; New utility crate
02:07=Tooltime!
02:07=This could come in handy...
02:07=Utilities!
02:07=Utilise this box
02:07=Watch out below
02:07=More utilities!
02:07=Tools for you!
02:07=This should be good!
02:07=Use this wisely
02:07=Ooo this box is heavy
02:07=You might need this

; Hog (%1) skips his turn
02:08=%1 is sooo boring...
02:08=%1 couldn't be bothered
02:08=%1 is one lazy hog
02:08=%1 is thoughtless
02:08=%1 gave up
02:08=You snooze you lose, %1
02:08=%1 shamelessly skips
02:08=%1 is really lazy
02:08=%1 needs a little more motivation
02:08=%1 is a pacifist
02:08=%1 has a breather
02:08=%1 has a rest
02:08=%1 chills out
02:08=%1 has no faith in his own abilities
02:08=%1 decides to do nothing at all
02:08=%1 lets the enemy destroy itself
02:08=%1 would be terrible at parties
02:08=%1 hides out
02:08=%1 has decided to pass on this opportunity
02:08=%1 decides the best thing he can do is...nothing
02:08=%1 is a big wuss
02:08=Buck Buck Buck, %1 is a chicken
02:08=%1 is looking a little yellow
02:08=%1 is a coward!
02:08=%1 is waiting for sudden death
02:08=%1 is not the fighting type
02:08=%1 is reconsidering his purpose in life
02:08=%1 was never much of a good shot anyway
02:08=%1 didn't want to join the army in the first place
02:08=Stop wasting our time, %1
02:08=I'm dissapointed in you, %1
02:08=Come on, you can do better than that %1
02:08=%1's will has broken
02:08=%1 apparently has better things to do
02:08=%1 is scared stiff
02:08=%1 has fallen asleep

; Hog (%1) hurts himself only
02:09=%1 should practice aiming!
02:09=%1 seems to hate himself
02:09=%1 is standing on the wrong side!
02:09=%1 makes like an emo
02:09=%1 was holding his weapon the wrong way around
02:09=%1 is a little sadistic
02:09=%1 is a masochist
02:09=%1 has no instinct of self-preservation
02:09=%1 messed up
02:09=%1 screwed up
02:09=That was a poor shot, %1
02:09=%1 is a little too careless with dangerous weapons
02:09=%1 should consider a change of career
02:09=Worst. Shot. Ever!
02:09=No no no %1, you shoot at the ENEMY!
02:09=%1 should only be destroying the enemy
02:09=%1 moves one step closer to suicide
02:09=%1 aids the enemy
02:09=That was stupid %1
02:09=%1 lives by the mantra of "no pain, no gain"
02:09=%1 is confused
02:09=%1 hurt itself in its confusion
02:09=%1 has a knack for embarrassing himself
02:09=%1 is a klutz!
02:09=%1 is clumsy
02:09=%1 shows the enemy what he's capable of
02:09=%1 can't be expected to be perfect all the time
02:09=Don't worry %1, pobody's nerfect
02:09=%1 totally did that on purpose
02:09=I won't tell anyone if you don't, %1
02:09=How embarrassing!
02:09=I'm sure nobody saw that %1
02:09=%1 needs to review his field manual
02:09=%1's weapon clearly malfunctioned

; Hog shot an home run (using the bat and another hog)
02:10=Home Run!
02:10=A bird, a plane, ...
02:10=That one is out!

; Weapon Categories
03:00=Timed Grenade
03:01=Timed Grenade
03:02=Ballistic Weapon
03:03=Guided Weapon
03:04=Gun (multiple shots)
03:05=Digging Tool
03:06=Action
03:07=Transport Utility
03:08=Proximity Bomb
03:09=Gun (multiple shots)
03:10=BOOM!
03:11=Bonk!
03:12=Martial Arts
03:13=UNUSED
03:14=Transport Utility
03:15=Airborne Attack
03:16=Airborne Attack
03:17=Digging Tool
03:18=Utility
03:19=Transport Utility
03:20=Action
03:21=Ballistic Weapon
03:22=Call me Indiana!
03:23=(Really) Martial Arts
03:24=The cake is NOT a lie!
03:25=Costume Kit
03:26=Juicy Grenade
03:27=Fiery Grenade
03:28=Ballistic Weapon
03:29=Ballistic Weapon
03:30=Airborne Attack
03:31=Remote Controlled Bomb
03:32=Temporary Effect
03:33=Temporary Effect
03:34=Temporary Effect
03:35=Temporary Effect
03:36=Temporary Effect
03:37=Temporary Effect
03:38=Gun (multiple shots)
03:39=Transport Utility
03:40=Incinerating Grenade
03:41=Huge fan of Squawks

; Weapon Descriptions (use | as line breaks)
04:00=Attack your enemies using a simple grenade.|It will explode once its timer reaches zero.|1-5: Set grenade's timer|Attack: Hold to throw with more power
04:01=Attack your enemies using a cluster bomb.|It will split into smaller bombs once its timer|reaches zero.|1-5: Set grenade's timer|Attack: Hold to throw with more power
04:02=Attack your enemies using a ballistic projectile|that might be influenced by wind.|Attack: Hold to shoot with more power
04:03=Launch an explosive bee that will lock on|the selected target. Don't shoot with full power|to improve its precision.|Cursor: Pick target|Attack: Hold to shoot with more power
04:04=Attack your enemy using a shotgun with two shots.|Thanks to its spread you don't need direct hits|to harm your opponents.|Attack: Shoot (multiple times)
04:05=Move underground! Use the pickhammer to drill|a hole into the ground and reach other areas.|Attack: Start or stop digging
04:06=Bored? No way to attack? Save your ammo?|No problem! Just skip your turn, coward!|Attack: Skip your turn without fighting
04:07=Bridge huge distances using timed shots with the|rope. Use your momentum to slide into other hogs|or drop grenades and other weapons on them.|Attack: Shoot or release the rope|Long Jump: Drop grenades or similar weapons
04:08=Keep your enemies away by dropping a mine in|narrow passages or right below their feet. Be|sure to retreat before you trigger it yourself!|Attack: Drop mine next to your feet
04:09=Not sure about your aiming? Use the Desert|Eagle to attack using up to four shots.|Attack: Shoot (multiple times)
04:10=Brute force is always an option. Drop this classic|explosive next to your enemies and retreat.|Attack: Drop dynamite next to your feet
04:11=Get rid of enemy hogs by batting them over|the map borders or into water. Or how about|knocking some mines to your friends?|Attack: Bat everything in front of you
04:12=Get close and personal to unleash the power of|this almost deadly martial arts technique.|Attack: Perform the Fire Punch
04:13=UNUSED
04:14=Fear of heights? Better grab a parachute.|It will unfold once|you fall too far and|save your hog from taking fall damage.|Attack: Unfold the parachute|Long Jump: Drop grenades or similar weapons
04:15=Call in an airplane to attack your enemies|using a bombing run.|Left/Right: Determine attack direction|Cursor: Select target region
04:16=Call in an airplane to drop several mines|in the target area.|Left/Right: Determine attack direction|Cursor: Select target region
04:17=Need shelter? Use the blow torch to dig|a tunnel into solid ground granting you|cover.|Attack: Start or stop digging
04:18=Need additional protection or want to pass|unpassable ground? Place some girders as you|like.|Left/Right: Select girder to place|Cursor: Place girder in a valid position
04:19=Used at the right moment teleportation can|be more powerful than almost all weapons as|it allows you to save hogs from dangerous|situations within seconds.|Cursor: Select target region
04:20=Allows you to play the current turn with|a different hog.|Attack: Enable switching hogs
04:21=Shoot a grenade-like projectile that will|release multiple bombs upon impact.|Attack: Shoot at full power
04:22=Not just for Indiana Jones! The whip is a|useful weapon in many situations. Especially|when you'd like to shove someone off a cliff.|Attack: Strike everything in front of you
04:23=If you have nothing to lose, this might be|quite handy. Sacrifice your hog by launching|him into a specific direction hurting everything|on his way and exploding at the end.|Attack: Launch the devastating and deadly attack
04:24=Happy Birthday! Launch this cake, let it walk right|next to your enemies and let them have an explosive|party. The cake is able to pass almost all terrain|but he might detonate earlier this way.|Attack: Start the cake or let it stop and explode
04:25=Use this disguise kit to get your enemies to jump|towards your hog (and into some gap or hole).|Attack: Use the kit and try to seduce another hog
04:26=Throw this juicy watermelon at your enemies. Once|the timer expires, it will split into several|explosive pieces.|1-5: Set watermelon's timer|Attack: Hold to shoot with more power
04:27=Let hellfire rain onto your opponents by using|this fiendish explosive. Don't get too close to|the explosion as smaller fires might last longer.|Attack: Hold to shoot with more power
04:28=Short time after launching this rocket, it will|start drilling through solid ground and explode|once its fuse is triggered or it resurfaces again.|Attack: Hold to shoot with more power
04:29=This is nothing for small kids! The ball gun fires|tons of small colored balls filled with explosives.|Attack: Shoot at full power|Up/Down: Continue aiming
04:30=Call in an airplane to launch a powerful napalm|strike. With proper aiming this attack can eradicate|huge parts of landscape including unlucky hogs|sitting there.|Left/Right: Determine attack direction|Cursor: Select target region
04:31=The RC plane is the ideal weapon to collect crates or|attack far away hogs. Either steer it into enemies or|drop some bombs first.|Attack: Launch the plane or drop bombs|Long Jump: Let the valkyries ride into battle|Up/Down: Steer the plane
04:32=Low gravity is more effective than any diet! Jump|higher and over greater distances or let your enemies|fly even further.|Attack: Activate
04:33=Sometimes you just need that little extra boost to|deal some more damage.|Attack: Activate
04:34=Can't touch me!|Attack: Activate
04:35=Sometimes time's running too fast. Grab some extra|seconds to finish your attack.|Attack: Activate
04:36=Well, sometimes you're just too bad in aiming. Get|some assistance using modern day technology.|Attack: Activate
04:37=Don't fear the daylight. It will just last one turn|but will enable you to absorb the damage you do to|other hogs.|Attack: Activate
04:38=The sniper rifle can be the most devastating weapon|in your whole arsenal, however it's very ineffective|at close quarters. The damage dealt increases with|the distance to its target.|Attack: Shoot (twice)
04:39=Fly to other parts of the map using the flying|saucer. This hard to master utility is able to|take you to almost any position on the battlefield.|Attack: Activate|Up/Left/Right: Apply force in one direction|Long Jump: Drop grenades or similar weapons
04:40=Set some ground on fire using this bottle filled|with (soon to be) burning liquid.|Attack: Hold to shoot with more power
04:41=The evidence nature might even top the flying|saucer. Birdy can carry your hog around and|drop eggs on your enemies!|Attack: Activate and drop eggs|Up/Left/Right: Flap in one direction

; Game goal strings
05:00=Game Modes
05:01=The following rules apply
05:02=Place the King: Pick a protected starting point for your King
05:03=Low Gravity: Watch your step
05:04=Invulnerability: Hogs are (almost) invulnerable
05:05=Vampirism: Hogs will be healed for the damage dealt
05:06=Karma: Hogs will be damaged for the damage dealt
05:07=Protect the King: Don't let your king die!
05:08=Place Hedgehogs: Place your hogs before the game starts
05:09=Artillery: Hogs can't walk to change position
05:10=Indestructible Terrain: Most weapons won't destroy terrain
05:11=Shared Ammo: All teams of the same color share their ammunition
05:12=Mine Timers: Mines will detonate after %1 second(s)
05:13=Mine Timers: Mines will detonate instantly
05:14=Mine Timers: Mines will detonate after 0 - 3 seconds
05:15=Damage Modifier: All weapons will do %1% damage
