
/******************************************************************************
 *
 * CSS-like definition of chat appearance
 *
 ******************************************************************************
 *
 * see http://doc.qt.nokia.com/4.5/richtext-html-subset.html#css-properties
 *
 * In the QTfrontend of hedgewars also display:none; will work for class names
 * that start with msg_ and .timestamp -  as long as they are referenced
 * directly and not within any class hierarchy.
 * Note: Will only effect new lines!
 *
 ******************************************************************************
 *
 * This file can be stored at different locations, but it will be read only
 * once, based on first file found in this order:
 *
 *    <userdir>/Data/css/chat.css ( create/see full path with: /saveStyleSheet )
 *    <datadir>/css/chat.css
 *    <internal default style-sheet>
 *
 *****************************************************************************/

/* valid color values
 *
 * transparent   (no color)
 *
 * colorname
 *   (a word that is the name of a valid color)
 *   available colors: http://www.w3.org/TR/SVG/types.html#ColorKeywords
 *   examples: red, steelblue, mintcream
 *
 * #RRGGBB
 *   (hexadecimal additive red-green-blue, max is ff, min 00, half is 80)
 *   examples: #ffff00 (yellow), #808080 (gray)
 *
 * rgb(r, g, b)
 *   (decimal additive red-green-blue, max is 255, min 0, half is 128)
 *   examples: rgb(135, 206, 250) (lightskyblue)
 */


/* links */
a { color: #c8c8ff; }

/* appearance of the whole chat */
/* body { color: white; background-color: #000000; } */

/* appearance of all elements in chat */
/* * { color: white; background-color: #000000; } */

/* nick names (they are also sometimes linked) */
.nick { text-decoration: none; }

/* various chat messages */
.msg_UserChat { color:#ffcc00; } /* chat of people who aren't your friends */
.msg_UserChat .nick { color:#ffec20; }
.msg_FriendChat { color: #b0ff08; }
.msg_FriendChat .nick { color: #30ff30; }
.msg_UserJoin { color: #c0c0c0; }
.msg_UserJoin .nick { color: #d0d0d0; }
.msg_UserLeave { color: #b8b8b8; }
.msg_UserLeave .nick { color: #c8c8c8; }
.msg_FriendJoin { font-weight: bold; color: #c0f0c0; }
.msg_FriendJoin .nick { color: #d8f0d8; }
.msg_FriendLeave { font-weight: bold; color: #ffe090; }
.msg_FriendLeave .nick { color: #f8e878; }
.msg_UserAction { color: #ff80ff; }
.msg_UserAction .nick { color: #ffa0ff;}
.msg_FriendAction { color: #ff00ff; }
.msg_FriendAction .nick { color: #ff30ff; }

/* uncomment next lines to disable join and leave messages of non-friends */
/* .msg_UserJoin { display:none; } */
/* .msg_UserLeave { display:none; } */

/* timestamps */
.timestamp {
    color: #e0d8e0;
    font-family: courier;
    font-size: 11px;
    vertical-align: center;
    /* uncomment next line to hide timestamps */
    /* display: none; */
}

/* .timestamp:hours { display: none; } */
.timestamp:seconds { display: none; }

/* you can also set timestamp style for different msg types */
.msg_FriendChat .timestamp { color: #ffffff; }

/* messages that contain your nickname */
.highlight {  }
.highlight .nick { color: #ff0000; } /* nicknames in highlighted messages */

/* system messages */
.msg_Error { color: #ff0000; }
.msg_Warning { color: #ff8000; }
.msg_Notice { color: #fefefe; }
.msg_Notice .nick { color: #ffffd0; }

