package org.hedgewars.hedgeroid;

import android.support.v4.app.FragmentActivity;

// TODO
public class WeaponsetCreatorActivity extends FragmentActivity {
    public static final String PARAMETER_EXISTING_WEAPONSETNAME="existingWeaponsetName";
}
