/******************************************************************************
 *
 * CSS definition of Qt frontend appearance: Default style.
 *
 ******************************************************************************
 *
 * see https://doc.qt.io/qt-5/style-reference.html
 *
 ******************************************************************************
 *
 * This file can be stored at different locations, but it will be read only
 * once, based on first file found in this order:
 *
 *    <userdir>/Data/css/qt.css
 *    <datadir>/css/qt.css
 *    <internal default style-sheet> (:/res/css/qt.css)
 *
 *****************************************************************************/

#infoButton 
{
border: transparent;
background: transparent; 
}
HWForm,QDialog {
background-image: url(":/res/Background.png");
background-position: bottom center;
background-repeat: repeat-x;
background-color: #141250;
}

* {
color: #ffcc00;
selection-background-color: #ffcc00;
selection-color: #00351d;
}

a { color:#c8c8ff; }

QLineEdit, QListWidget, QListView, QTableView, QTableWidget, QTextBrowser, QSpinBox, QComboBox,
QComboBox QAbstractItemView, QPlainTextEdit, QMenu::item, #labelLikeLineEdit {
background-color: rgba(13, 5, 68, 70%);
}

VertScrArea, QGraphicsView {
border-style: solid; border-width: 2px; border-color: #cca300; border-radius: 3px;
}
#gameStatsView {
border-color: #332816;
}

QSplitter::handle {
background-image: url(":/res/splitter.png");
background-clip: content;
}
QSplitter::handle:horizontal {
width: 7px;
background-repeat: repeat-y;
}
QSplitter::handle:vertical {
height: 7px;
background-repeat: repeat-x;
}

QComboBox::separator {
border: solid; border-width: 3px; border-color: #ffcc00;
}

QPushButton, QListWidget, QListView, QTableView, QTableWidget, QLineEdit,
QTextBrowser, QSpinBox, QToolBox, QComboBox, QPlainTextEdit,
QComboBox QAbstractItemView, IconedGroupBox,
.QGroupBox, #gameStackContainer, TeamSelWidget, SelWeaponWidget,
QTabWidget::pane, QTabBar::tab, #mapPreview, #labelLikeLineEdit {
border: solid;
border-width: 3px;
border-color: #ffcc00;
}

QPushButton:hover, QLineEdit:hover, QListWidget:hover, QListView:hover,
QSpinBox:hover, QToolBox:hover, QComboBox:hover {
border-color: yellow;
}

TeamShowWidget QPushButton {
icon-size: 48px;
text-align: left;
background-color: #0d0544;
color: orange;
font: bold;
border-width: 2px;
margin: 6px 0px 6px 0px;
}
TeamShowWidget QPushButton:disabled {
color: #a0a0a0;
}
TeamShowWidget QPushButton:pressed {
background-color: #362c7a;
}

QToolButton {
background-color: #11084A;
}

QToolButton:hover {
background-color: #150A61;
}

QLineEdit, QListWidget, QListView, QTableView, QTableWidget, QTextBrowser,
QSpinBox, QToolBox, QPlainTextEdit, QToolButton, #mapPreview, #labelLikeLineEdit {
border-radius: 10px;
}

#mapPreview {
background-color: #0d0544;
}
#mapPreview:disabled{
border-color: #a0a0a0;
background-color: #0d0544;
color: #ffffff;
}

QLineEdit, QLabel, QHeaderView, QListWidget, QListView, QTableView,
QTableWidget, QSpinBox, QToolBox::tab, QComboBox, QComboBox QAbstractItemView,
IconedGroupBox, .QGroupBox, #gameStackContainer, TeamSelWidget,
SelWeaponWidget, QCheckBox, QRadioButton, QPushButton, QPlainTextEdit,
#mapName {
font: bold 13px;
}
.QLabel{
background-color: transparent;
}
SelWeaponWidget QTabWidget::pane, SelWeaponWidget QTabBar::tab:selected {
background-position: bottom center;
background-repeat: repeat-x;
background-color: #000000;
}
.QGroupBox, #gameStackContainer, TeamSelWidget, SelWeaponWidget {
background-position: bottom center;
background-repeat: repeat-x;
border-radius: 16px;
background-color: rgba(13, 5, 68, 70%);
padding: 6px;
margin-top: 4px;
}
/*  Experimenting with PaintOnScreen and border-radius on IconedGroupBox children didn't work out well
IconedGroupBox QComboBox, IconedGroupBox QPushButton, IconedGroupBox QLineEdit,
IconedGroupBox QSpinBox {
border-radius: 0;
}
IconedGroupBox, IconedGroupBox *, QTabWidget::pane, QTabBar::tab:selected, QToolBox::tab QWidget{" */
IconedGroupBox, QTabWidget::pane, QTabBar::tab:selected, QToolBox::tab QWidget{
background-color: #130f2c;
}

QTabWidget::pane {
border-radius: 8px;
border-top-left-radius: 0px;
}

GameCFGWidget {
border: none;
}

QPushButton {
border-radius: 8px;
background-origin: margin;
background-position: top left;
background-color: rgba(18, 42, 5, 70%);
}

QPushButton:pressed, QToolButton:pressed {
border-color: white;
background-color: #344b1e;
color: white;
}

QPushButton:focus {
outline: none;
}

QHeaderView {
background-color: #00351d;
border: solid;
border-bottom-width: 3px;
border-top-width: 0px;
border-left-width: 0px;
border-right-width: 0px;
border-color: #ffcc00;
}
QHeaderView::section {
border-left-width: 1px;
border-right-width: 1px;
border-top-width: 0;
border-bottom-width: 0px;
border-color: #005F35;
border-style: solid;
background-color: #00351d;
padding: 4px;
}
QHeaderView::section:pressed {
background-color: #005A33;
border-color: #FFFFFF;
}
QHeaderView::up-arrow {
image: url(":/res/sort_up.png");
}
QHeaderView::down-arrow{
image: url(":/res/sort_down.png");
}

QTableView, QTableWidget {
alternate-background-color: #2f213a;
gridline-color: transparent;
}
QTabWidget::pane { top: -2px; }
QTabBar::tab {
border-radius: 0;
border-top-left-radius: 6px;
border-top-right-radius: 6px;
padding: 3px;
background-color: #00351d;
color: #ffcc00;
}
QTabBar::tab:selected {
border-bottom-color: #0d0544;
border-bottom-width: 0;
}
QSpinBox::up-button{
background: transparent;
width: 16px;
height: 10px;
padding-top: 1px;
}

QSpinBox::up-arrow {
image: url(":/res/spin_up.png");
}
QSpinBox::up-arrow:disabled {
image: url(":/res/spin_up_disabled.png");
}

QSpinBox::down-arrow {
image: url(":/res/spin_down.png");
}
QSpinBox::down-arrow:disabled {
image: url(":/res/spin_down_disabled.png");
}

QSpinBox::down-button {
background: transparent;
width: 16px;
height: 10px;
padding-bottom: 1px;
}

QSpinBox, QLineEdit {
padding: 3px;
min-height: 18px;
}
QComboBox {
border-radius: 10px;
padding: 3px;
height: 18px;
}
QComboBox:pressed{
border-color: white;
}
QComboBox::drop-down{
border: transparent;
width: 25px;
}
QComboBox::down-arrow {
image: url(":/res/dropdown.png");
}
QComboBox::down-arrow:disabled {
image: url(":/res/dropdown_disabled.png");
}

VertScrArea {
background-position: bottom center;
background-repeat: repeat-x;
}

IconedGroupBox {
border-radius: 16px;
padding: 2px;
}

QGroupBox::title{
subcontrol-origin: margin;
subcontrol-position: top left;
text-align: left;
left: 15px;
}

QCheckBox::indicator:checked{
image: url(":/res/checked.png");
}
QCheckBox::indicator:checked:hover{
image: url(":/res/checkedHover.png");
}
QCheckBox::indicator:checked:pressed{
image: url(":/res/checkedPressed.png");
}
QCheckBox::indicator:unchecked{
image: url(":/res/unchecked.png");
}
QCheckBox::indicator:unchecked:hover{
image: url(":/res/uncheckedHover.png");
}
QCheckBox::indicator:unchecked:pressed{
image: url(":/res/uncheckedPressed.png");
}

QRadioButton::indicator:checked{
image: url(":/res/radioButtonChecked.png");
}
QRadioButton::indicator:checked:hover{
image: url(":/res/radioButtonCheckedHover.png");
}
QRadioButton::indicator:checked:pressed{
image: url(":/res/radioButtonCheckedPressed.png");
}
QRadioButton::indicator:unchecked{
image: url(":/res/radioButtonUnchecked.png");
}
QRadioButton::indicator:unchecked:hover{
image: url(":/res/radioButtonUncheckedHover.png");
}
QRadioButton::indicator:unchecked:pressed{
image: url(":/res/radioButtonUncheckedPressed.png");
}

.QWidget{
background: transparent;
}

QTabWidget::pane {
border-top-width: 2px;
}

QMenu{
background-color: #ffcc00;
margin: 3px;
}
QMenu::item {
background-color: #0d0544;
border: 1px solid transparent;
font: bold;
padding: 2px 25px 2px 20px;
}
QMenu::item:selected {
background-color: #2d2564;
}
QMenu::indicator {
width: 16px;
height: 16px;
}
QMenu::indicator:non-exclusive:checked{
image: url(":/res/checked.png");
}
QMenu::indicator:non-exclusive:unchecked{
image: url(":/res/unchecked.png");
}

QToolTip{
background-color: #0d0544;
border: 1px solid #ffcc00;
}

:disabled{
color: #a0a0a0;
border-color: #a0a0a0;
}
QListWidget:item:selected:disabled, QListView:item:selected:disabled{
color: rgba(13, 5, 68, 70%);
background-color: #a0a0a0;
}
SquareLabel, ItemNum {
background-color: #000000;
}

QSlider::groove::horizontal {
height: 2px;
margin: 2px 0px;
background-color: #ffcc00;
}
QSlider::groove::horizontal:disabled {
background-color: #a0a0a0;
}

QSlider::handle::horizontal {
border: 0px;
margin: -8px 0px;
background-color: #ffcc00;
width: 12px;
height: 6px;
border-radius: 3px;
}
QSlider::handle::horizontal:disabled {
background-color: #a0a0a0;
}

HatButton, ThemeButton {
text-align: left;
}

#hatList, #hatList:hover, #themeList, #themeList:hover {
border-color: #F6CB1C;
}

QScrollBar:vertical {
border: none;
color: #FFD902;
background: #00321c;
width: 15px;
margin: 17px 0 17px 0;
}
QScrollBar:horizontal {
border: none;
color: #FFD902;
background: #00321c;
height: 15px;
margin: 0 17px 0 17px;
}
QScrollBar::handle:vertical {
background: #00321c;
border: 1px solid #005F35;
border-radius: 2px;
min-height: 20px;
}
QScrollBar::handle:horizontal {
background: #00321c;
border: 1px solid #005F35;
border-radius: 2px;
min-width: 20px;
}
QScrollBar::handle:pressed {
background: #005a33;
border-color: #FFFFFF;
}
QScrollBar::add-line, QScrollBar::sub-line {
border: 1px solid #005F35;
border-radius: 2px;
background: #00321c;
subcontrol-origin: margin;
}
QScrollBar::add-line:vertical {
height: 15px;
subcontrol-position: bottom;
}
QScrollBar::sub-line:vertical {
height: 15px;
subcontrol-position: top;
}
QScrollBar::add-line:horizontal {
width: 15px;
subcontrol-position: right;
}
QScrollBar::sub-line:horizontal {
width: 15px;
subcontrol-position: left;
}
QScrollBar::add-line:pressed, QScrollBar::sub-line:pressed {
background: #005a33;
border-color: #FFFFFF;
}

QScrollBar::up-arrow {
background-image: url(":/res/scroll_up.png");
}
QScrollBar::down-arrow {
background-image: url(":/res/scroll_down.png");
}
QScrollBar::left-arrow {
background-image: url(":/res/scroll_left.png");
}
QScrollBar::right-arrow {
background-image: url(":/res/scroll_right.png");
}
QScrollBar::add-page, QScrollBar::sub-page {
background: #00190F;
}
QScrollBar::add-page:pressed, QScrollBar::sub-page:pressed {
background: #008751;
}

#hatList, #themeList {
border-color: #F6CB1C;
border-width: 3px;
border-style: solid;
border-radius: 10px;
border-top-left-radius: 0px;
}

#hatList::item, #themeList::item {
background-color: #11084A;
padding: 4px;
border-radius: 10px;
color: #ffcc00 !important;
font: 8px;
border-width: 2px;
border-color: #11084A;
}

#hatList::item:hover, #themeList::item:hover {
background-color: #150A61;
}

#hatList::item:selected, #themeList::item:selected {
background-color: #150A61;
}

.QPushButton, .QPushButtonWithSound {
padding: 3px 5px;
}

#gameCfgWidgetTabs {
border-radius: 16px;
border-top-left-radius: 0px;
}

TeamSelWidget, #gameStackContainer, #GBoxOptions {
border-radius: 10px;
}

PageMultiplayer TeamSelWidget {
min-height: 500px;
}

QProgressBar {
border: 3px solid #FFCC00;
background-color: #150A61;
border-radius: 5px;
text-align: center;
color: #FFFFFF;
}
QProgressBar::chunk {
background-color: #FF9B00;
margin: 2px;
}
#keyBinderScrollArea {
background: #130F2A;
}
#chatContainer {
border-width: 0px;
background-color: #ffcc00;
border-radius: 10px;
}
#chatText {
background-color: rgb(23, 11, 54);
border-width: 0px;
}
#trainingList {
border-style: none;
padding-top: 6px;
}
#trainingList::item
{
padding-top: 2px;
padding-bottom: 2px;
}
ToggleButtonWidget
{
padding: 0px;
}
